package com.supervision.contoller;

import cn.hutool.core.io.resource.ResourceUtil;
import cn.hutool.core.util.StrUtil;
import com.supervision.dto.R;
import com.supervision.dto.robot.RobotTalkDTO;
import com.supervision.model.RobotTalkReq;
import com.supervision.model.dify.StreamResponse;
import com.supervision.service.DifyService;
import com.supervision.service.IChatService;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.http.codec.ServerSentEvent;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import reactor.core.publisher.Flux;

import java.io.IOException;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/chat")
@RequiredArgsConstructor
public class ChatController {

    private final IChatService chatService;
    private final DifyService difyService;

    @PostMapping("/talk")
    public R<RobotTalkDTO> talk(@RequestParam("file") MultipartFile multipartFile, @ModelAttribute RobotTalkReq robotTalkReq) {
        RobotTalkDTO talk = chatService.talk(multipartFile,robotTalkReq);
        return R.ok(talk);
    }

    @PostMapping("/asr")
    public R<String> asr(@RequestParam("file") MultipartFile file) throws IOException {
        return R.ok(chatService.asr(file));
    }

    @GetMapping("/talkList")
    public List talkList(String sessionId) {
        return new ArrayList();
    }

    @GetMapping("/getAudio")
    public void getAudio(HttpServletResponse response,
                         @RequestParam("audioId")String audioId) throws IOException {
         chatService.getAudio(response,audioId);
    }

    @GetMapping(value="/stream",produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    public Flux<ServerSentEvent<Map<String, String>>> test2(@RequestParam("query") String query) {
        return chatService.streamingMessage(query);
    }

    @GetMapping(value = "/webflux",produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    public Flux<String> chatWebFlux() {
        String string = ResourceUtil.readUtf8Str("classpath:static/test.txt");
        String[] stringArray = StrUtil.split(string, 1);
        return Flux.just(stringArray).delayElements(Duration.ofMillis(50));
    }
}
