package com.supervision.service;

import com.alibaba.fastjson.JSON;
import com.supervision.dto.paddlespeech.res.TtsResultDTO;
import com.supervision.dto.robot.AnswerInfo;
import com.supervision.model.dify.DIFYChatReqInputVO;
import com.supervision.model.dify.DifyChatReqVO;
import com.supervision.model.dify.StreamResponse;
import com.supervision.util.TtsUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.http.codec.ServerSentEvent;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

@Slf4j
@Service
@RequiredArgsConstructor
public class DifyService {
    @Value("${dify.url}")
    private String difyUrl;
    @Value("${dify.app-auth}")
    private String difyAppAuth;
    private final WebClient webClient;
    Map<String, String> voiceCache = new HashMap<>();

    /**
     * 流式调用dify.
     *
     * @param query 查询文本
     * @return Flux 响应流
     */
    public Flux<ServerSentEvent<Map<String, String>>> streamingMessage(String query) {
        DifyChatReqVO difyChatReqVO = new DifyChatReqVO();
        difyChatReqVO.setUser("admin");
        DIFYChatReqInputVO inputs = new DIFYChatReqInputVO();
        difyChatReqVO.setQuery(query);
//        difyChatReqVO.setQuery("尽可能详细的介绍一下勐赫小镇的医疗服务");
        difyChatReqVO.setInputs(inputs);
        StringBuilder sentence = new StringBuilder();

        return webClient.post()
                .uri(difyUrl)
                .headers(httpHeaders -> {
                    httpHeaders.setContentType(MediaType.APPLICATION_JSON);
                    httpHeaders.setBearerAuth(difyAppAuth);
                })
                .bodyValue(JSON.toJSONString(difyChatReqVO))
                .retrieve()
                .bodyToFlux(StreamResponse.class)
                .map(response -> {
                    Map<String, String> map = new HashMap<>();
                    map.put("event", response.getEvent());
                    if (response.getEvent().equals("message") && response.getAnswer() != null) {
                        TtsResultDTO ttsResultDTO = TtsUtil.ttsTransform(response.getAnswer());
                        String voiceBaseId = UUID.randomUUID().toString();
                        voiceCache.put(voiceBaseId, ttsResultDTO.getAudio());
                        map.put("audioId", voiceBaseId);
                    }
                    return ServerSentEvent.builder(map).build();

//                    if (response.getEvent().equals("message") && response.getAnswer() != null) {
//                        //遍历answer中的每一个字符，判断是否为标点符号，如果是，说明是句子的结尾，将标点符号前的文本拼接到sentence中，并打印，然后清空sentence，如果标点符号后还有文本，将文本拼接到sentence中
//                        for (char ch : response.getAnswer().toCharArray()) {
//                            sentence.append(ch);
//                            if (ch == '。' || ch == '！' || ch == '？' || ch == '，' || ch == '、' || ch == '‘' || ch == '’' || ch == '“' || ch == '”') { // Check for punctuation marks
//                                log.info(sentence.toString());
//                                sentence.setLength(0); // Clear the sentence
//                            }
//                        }
//                    }
//                    if (response.getEvent().equals("message_end") && !sentence.isEmpty()) {
//                        log.info(sentence.toString());
//                    }
//                    Map<String, String> map;
//                    map = Map.of("event", response.getEvent(), "audioId", sentence.toString());
//                    return ServerSentEvent.builder(map).build();
                });
    }
}
