package com.supervision.util;

import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.supervision.dto.dify.ChatResDTO;
import com.supervision.model.dify.DIFYChatReqInputVO;
import com.supervision.model.dify.DifyChatReqVO;
import com.supervision.model.dify.StreamResponse;
import io.micrometer.common.util.StringUtils;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.hc.client5.http.ClientProtocolException;
import org.apache.hc.client5.http.classic.methods.HttpPost;
import org.apache.hc.client5.http.impl.classic.CloseableHttpClient;
import org.apache.hc.client5.http.impl.classic.HttpClients;
import org.apache.hc.core5.http.HttpEntity;
import org.apache.hc.core5.http.HttpHeaders;
import org.apache.hc.core5.http.HttpStatus;
import org.apache.hc.core5.http.io.entity.EntityUtils;
import org.apache.hc.core5.http.io.entity.StringEntity;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;

import java.nio.charset.StandardCharsets;

@Component
@Slf4j
@Service
public class DifyApiUtil {

    @Value("${dify.url}")
    private String difyUrl;
    @Value("${dify.app-auth}")
    private String difyAppAuth;
    @Resource
    private WebClient webClient;
    /**
     * 流式调用dify.
     *
     * @param query          查询文本
     * @return Flux 响应流
     */
    public Flux<StreamResponse> streamingMessage(String query) {
        DifyChatReqVO difyChatReqVO = new DifyChatReqVO();
        difyChatReqVO.setUser("admin");
        DIFYChatReqInputVO inputs = new DIFYChatReqInputVO();
        difyChatReqVO.setQuery("你好");
        difyChatReqVO.setInputs(inputs);

        return webClient.post()
                .uri(difyUrl)
                .headers(httpHeaders -> {
                    httpHeaders.setContentType(MediaType.APPLICATION_JSON);
                    httpHeaders.setBearerAuth(difyAppAuth);
                })
                .bodyValue(JSON.toJSONString(difyChatReqVO))
                .retrieve()
                .bodyToFlux(StreamResponse.class);
    }

    private boolean shouldInclude(StreamResponse streamResponse) {
        // 示例：只要message节点的数据和message_end节点的数据
        return streamResponse.getEvent().equals("message")
                || streamResponse.getEvent().equals("message_end");
    }

    public ChatResDTO chat(DifyChatReqVO difyChatReqVO) {
        ChatResDTO execute;
        try (CloseableHttpClient httpClient = HttpClients.createDefault()) {
            HttpPost httpPost = new HttpPost(difyUrl);
            httpPost.setHeader(HttpHeaders.AUTHORIZATION, difyAppAuth);
            httpPost.setHeader(HttpHeaders.CONTENT_TYPE, "application/json");
            log.info("发起对话：{}", difyChatReqVO);
            StringEntity entity = new StringEntity(com.alibaba.fastjson.JSONObject.toJSON(difyChatReqVO).toString(), StandardCharsets.UTF_8);
            httpPost.setEntity(entity);
            execute = httpClient.execute(httpPost, response -> {
                final int status = response.getCode();
                if (status >= HttpStatus.SC_SUCCESS && status < HttpStatus.SC_REDIRECTION) {
                    final HttpEntity responseEntity = response.getEntity();
                    String responseStr = EntityUtils.toString(responseEntity);
                    ChatResDTO chatResDTO = JSONUtil.toBean(responseStr, ChatResDTO.class);
                    log.info("DIFY返回：{}", chatResDTO.getAnswer());
                    return chatResDTO;
                } else {
                    log.info("responseEntity {}", EntityUtils.toString(response.getEntity()));
                    throw new ClientProtocolException("Unexpected response status: " + status);
                }
            });
        } catch (Exception e) {
            log.error("发起对话失败", e);
            throw new RuntimeException("发起对话失败");
        }
        return execute;
    }

}
